"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
require("jest-extended");
var Future_1 = require("./Future");
describe("Future", function () {
    describe("engage", function () {
        test("runs action provided in constructor with reject/resolve callbacks", function () {
            var actionSpy = jasmine.createSpy("futureAction");
            var rejectSpy = jasmine.createSpy("rejectSpy");
            var resolveSpy = jasmine.createSpy("resolveSpy");
            new Future_1.default(actionSpy).engage(rejectSpy, resolveSpy);
            expect(actionSpy).toHaveBeenCalledWith(rejectSpy, resolveSpy);
            expect(rejectSpy).not.toHaveBeenCalled();
            expect(resolveSpy).not.toHaveBeenCalled();
        });
        test("invokes reject when action throws exception", function () {
            var actionSpy = jasmine.createSpy("futureAction").and.throwError("forced error");
            var rejectSpy = jasmine.createSpy("rejectSpy");
            var resolveSpy = jasmine.createSpy("resolveSpy");
            new Future_1.default(actionSpy).engage(rejectSpy, resolveSpy);
            expect(actionSpy).toHaveBeenCalledWith(rejectSpy, resolveSpy);
            expect(rejectSpy).toHaveBeenCalledWith(new Error("forced error"));
            expect(resolveSpy).not.toHaveBeenCalled();
        });
        test("resolves with expected value on success", function () {
            var action = function (_reject, resolve) {
                resolve("my value");
            };
            var rejectSpy = jasmine.createSpy("rejectSpy");
            var resolveSpy = jasmine.createSpy("resolveSpy");
            new Future_1.default(action).engage(rejectSpy, resolveSpy);
            expect(resolveSpy).toHaveBeenCalledWith("my value");
        });
    });
    describe("toPromise", function () {
        test("returns a promise which can be chained with return value", function () {
            var actionSpy = jasmine.createSpy("futureAction");
            var prom = new Future_1.default(actionSpy).toPromise();
            expect(prom instanceof Promise).toBeTrue();
            expect(actionSpy).toHaveBeenCalledWith(jasmine.any(Function), jasmine.any(Function));
        });
        test("runs then result with value", function (done) {
            var action = function (_reject, resolve) {
                resolve("my value");
            };
            new Future_1.default(action)
                .toPromise()
                .then(function (val) {
                expect(val).toEqual("my value");
                done();
            })
                .catch(function () { return fail("shouldn't fail"); });
        });
        test("catches error during exception", function (done) {
            var action = function (reject) {
                reject(new Error("error value"));
            };
            new Future_1.default(action).toPromise().catch(function (val) {
                expect(val).toEqual(new Error("error value"));
                done();
            });
        });
    });
    describe("map", function () {
        test("converts value to new value", function (done) {
            var action = function (_reject, resolve) {
                resolve("my value");
            };
            new Future_1.default(action)
                .map(function (val) {
                expect(val).toEqual("my value");
                return "changed value";
            })
                .engage(function () { return fail("engage reject callback should not have been invoked"); }, function (val) {
                expect(val).toEqual("changed value");
                done();
            });
        });
        test("does not run map when first action fails", function (done) {
            var action = function (reject) {
                reject(new Error("error value"));
            };
            new Future_1.default(action)
                .map(function () {
                fail("map should not be called when action fails");
            })
                .engage(function (error) {
                expect(error).toEqual(new Error("error value"));
                done();
            }, function () { return fail("success callback should not be invoked on failure"); });
        });
    });
    describe("flatMap", function () {
        test("maps futures together", function (done) {
            var action = function (_reject, resolve) {
                resolve("my value");
            };
            new Future_1.default(action)
                .flatMap(function (val) {
                expect(val).toEqual("my value");
                return new Future_1.default(function (_reject, resolve) {
                    resolve("my new future value");
                });
            })
                .engage(function () { return fail("Failure handler should not be invoked when handleWith succeeds"); }, function (val) {
                expect(val).toEqual("my new future value");
                done();
            });
        });
        test("does not invoke flatMap when first action fails", function (done) {
            var action = function (reject) {
                reject(new Error("error value"));
            };
            new Future_1.default(action)
                .flatMap(function () {
                fail("flatMap should not be invoked if first action fails");
                return Future_1.default.of("does not matter");
            })
                .engage(function (err) {
                expect(err).toEqual(new Error("error value"));
                done();
            }, function () { return fail("resolve handler should not be called when action failed"); });
        });
        test("invokes reject handler if flatMap fails", function (done) {
            var action = function (_reject, resolve) {
                resolve("my value");
            };
            new Future_1.default(action)
                .flatMap(function (val) {
                expect(val).toEqual("my value");
                return new Future_1.default(function (reject) {
                    reject(new Error("flatMap error value"));
                });
            })
                .engage(function (err) {
                expect(err).toEqual(new Error("flatMap error value"));
                done();
            }, function () { return fail("resolve handler should not be called when action failed"); });
        });
    });
    describe("handleWith", function () {
        test("allows mapping of error to new result", function (done) {
            var action = function (reject) {
                reject(new Error("failure content"));
            };
            new Future_1.default(action)
                .handleWith(function (err) {
                expect(err).toEqual(new Error("failure content"));
                return new Future_1.default(function (_reject, resolve) {
                    resolve("my value");
                });
            })
                .engage(function () { return fail("Failure handler should not be invoked when handleWith succeeds"); }, function (val) {
                expect(val).toEqual("my value");
                done();
            });
        });
        test("does not get run if action succeeds", function (done) {
            var action = function (_reject, resolve) {
                resolve("my value");
            };
            new Future_1.default(action)
                .handleWith(function () {
                fail("handleWith should not be called during success case");
                return Future_1.default.of("does not matter");
            })
                .engage(function () { return fail("reject handler shouldnt be invoked "); }, function (val) {
                expect(val).toEqual("my value");
                done();
            });
        });
    });
    describe("errorMap", function () {
        test("converts error to new error value", function (done) {
            var action = function (reject) {
                reject(new Error("error value"));
            };
            new Future_1.default(action)
                .errorMap(function (err) {
                expect(err).toEqual(new Error("error value"));
                return new Error("mapped error");
            })
                .engage(function (err) {
                expect(err).toEqual(new Error("mapped error"));
                done();
            }, function () { return fail("resolve callback should not be invoked on error"); });
        });
        test("does not get invoked when action succeeds", function (done) {
            var action = function (_reject, resolve) {
                resolve("my value");
            };
            new Future_1.default(action)
                .errorMap(function (err) {
                expect(err).toEqual(new Error("error value"));
                return new Error("mapped error");
            })
                .engage(function () { return fail("engage reject callback should not have been invoked"); }, function (val) {
                expect(val).toEqual("my value");
                done();
            });
        });
    });
    describe("tryF", function () {
        test("resolves with the return value of the provided function", function () {
            var fnToValue = jasmine.createSpy("fnToValue").and.returnValue("test");
            Future_1.default.tryF(fnToValue).engage(function (e) { return fail(e.message); }, function (result) {
                expect(result).toEqual("test");
                expect(fnToValue).toHaveBeenCalledWith();
            });
        });
        test("rejects if the provided function throws an exception", function () {
            var fnToException = jasmine.createSpy("fnToPromise").and.throwError("forced failure");
            Future_1.default.tryF(fnToException).engage(function (error) {
                expect(error.message).toEqual("forced failure");
                expect(fnToException).toHaveBeenCalledWith();
            }, function () { return fail("should reject when function throws an exception"); });
        });
    });
    describe("tryP", function () {
        test("wraps returned promise in then/catch if returned", function () {
            var fauxCatch = jasmine.createSpy("fauxCatch");
            var fauxThen = jasmine.createSpy("fauxThen").and.returnValue({
                catch: fauxCatch,
            });
            var fauxPromise = jasmine.createSpy("fauxPromise").and.returnValue({
                then: fauxThen,
            });
            var rejectSpy = jasmine.createSpy("rejectSpy");
            var resolveSpy = jasmine.createSpy("resolveSpy");
            Future_1.default.tryP(fauxPromise).engage(rejectSpy, resolveSpy);
            expect(fauxPromise).toHaveBeenCalledWith();
            expect(fauxThen).toHaveBeenCalledWith(resolveSpy);
            expect(fauxCatch).toHaveBeenCalledWith(rejectSpy);
        });
        test("rejects if function that creates the promise throws an exception", function () {
            var fnToPromise = jasmine.createSpy("fnToPromise").and.throwError("forced failure");
            Future_1.default.tryP(fnToPromise).engage(function (error) {
                expect(error.message).toEqual("forced failure");
                expect(fnToPromise).toHaveBeenCalledWith();
            }, function () { return fail("should reject when function throws an exception"); });
        });
    });
    describe("of", function () {
        test("resolves Future with provided value", function (done) {
            Future_1.default.of("fixed value").engage(function () { return fail("reject handler should never be called when using Future.of"); }, function (val) {
                expect(val).toEqual("fixed value");
                done();
            });
        });
    });
    describe("reject", function () {
        test("rejects Future with provided value", function (done) {
            Future_1.default.reject(new Error("fixed error")).engage(function (err) {
                expect(err).toEqual(new Error("fixed error"));
                done();
            }, function () { return fail("resolve handler should never be called when using Future.reject"); });
        });
    });
    describe("encase", function () {
        test("resolves with value of calling method", function () {
            var fn = jasmine.createSpy("fn").and.returnValue("test");
            var val = "provided value";
            Future_1.default.encase(fn, val).engage(function (e) { return fail(e); }, function (value) {
                expect(value).toEqual("test");
                expect(fn).toHaveBeenCalledWith("provided value");
            });
        });
        test("rejects if function throws an exception", function () {
            var fn = jasmine.createSpy("fn").and.throwError("forced failure");
            var val = "provided value";
            Future_1.default.encase(fn, val).engage(function (e) {
                expect(e.message).toEqual("forced failure");
            }, function () { return fail("Should not resolve when method fails"); });
        });
    });
    describe("gather2", function () {
        test("resolves both futures provided", function () {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.of("second future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.gather2(f1, f2).engage(function (e) { return fail(e); }, function (val) {
                expect(val).toBeArrayOfSize(2);
                expect(val[0]).toEqual("first future value");
                expect(val[1]).toEqual("second future value");
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
            });
        });
        test("rejects with error when first future fails", function () {
            var f1 = Future_1.default.reject(new Error("first failed future"));
            var f2 = Future_1.default.of("second future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.gather2(f1, f2).engage(function (err) {
                expect(err).toEqual(new Error("first failed future"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).not.toHaveBeenCalled();
            }, function () { return fail("resolve callback should not be invoked when any of the futures fail"); });
        });
        test("rejects with error when any subsequent future fails", function () {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.reject(new Error("second failure message"));
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.gather2(f1, f2).engage(function (err) {
                expect(err).toEqual(new Error("second failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
            }, function () { return fail("resolve callback should not be invoked when any of the futures fail"); });
        });
        test("rejects with first error that occured when multiple errors", function (done) {
            var f1 = new Future_1.default(function (reject) {
                setTimeout(function () {
                    reject(new Error("first failure message"));
                });
            });
            var f2 = Future_1.default.reject(new Error("second failure message"));
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.gather2(f1, f2).engage(function (err) {
                expect(err).toEqual(new Error("second failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                done();
            }, function () { return fail("resolve callback should not be invoked when any of the futures fail"); });
        });
        test("engages all futures without waiting", function (done) {
            var f1 = new Future_1.default(function (_, resolve) {
                setTimeout(function () { return resolve("first future value"); });
            });
            var f2 = Future_1.default.of("second future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.gather2(f1, f2).engage(function (e) { return fail(e); }, function (val) {
                expect(val).toBeArrayOfSize(2);
                expect(val[0]).toEqual("first future value");
                expect(val[1]).toEqual("second future value");
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                done();
            });
        });
    });
    describe("gather3", function () {
        test("resolves with array of values when all 3 Futures succeed", function () {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.of("second future value");
            var f3 = Future_1.default.of("third future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            Future_1.default.gather3(f1, f2, f3).engage(function (e) { return fail(e); }, function (val) {
                expect(val).toBeArrayOfSize(3);
                expect(val[0]).toEqual("first future value");
                expect(val[1]).toEqual("second future value");
                expect(val[2]).toEqual("third future value");
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                expect(f3.engage).toHaveBeenCalled();
            });
        });
        test("rejects with error when first future fails", function () {
            var f1 = Future_1.default.reject(new Error("first failed future"));
            var f2 = Future_1.default.of("second future value");
            var f3 = Future_1.default.of("third future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            Future_1.default.gather3(f1, f2, f3).engage(function (err) {
                expect(err).toEqual(new Error("first failed future"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).not.toHaveBeenCalled();
                expect(f3.engage).not.toHaveBeenCalled();
            }, function () { return fail("resolve callback should not be invoked when any of the futures fail"); });
        });
        test("rejects with error when any subsequent future fails", function () {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.reject(new Error("second failure message"));
            var f3 = Future_1.default.of("third future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            Future_1.default.gather3(f1, f2, f3).engage(function (err) {
                expect(err).toEqual(new Error("second failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                expect(f3.engage).not.toHaveBeenCalled();
            }, function () { return fail("resolve callback should not be invoked when any of the futures fail"); });
        });
        test("rejects with first error that occured when multiple errors", function (done) {
            var f1 = new Future_1.default(function (reject) {
                setTimeout(function () {
                    reject(new Error("first failure message"));
                });
            });
            var f2 = Future_1.default.of("second future result");
            var f3 = Future_1.default.reject(new Error("third failure message"));
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            Future_1.default.gather3(f1, f2, f3).engage(function (err) {
                expect(err).toEqual(new Error("third failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                expect(f3.engage).toHaveBeenCalled();
                done();
            }, function () { return fail("resolve callback should not be invoked when any of the futures fail"); });
        });
        test("engages all futures without waiting", function (done) {
            var f1 = new Future_1.default(function (reject) {
                setTimeout(function () { return reject(new Error("first failure message")); });
            });
            var f2 = new Future_1.default(function (reject) {
                setTimeout(function () { return reject(new Error("second failure message")); });
            });
            var f3 = Future_1.default.of("third future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            Future_1.default.gather3(f1, f2, f3).engage(function (err) {
                expect(err).toEqual(new Error("first failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                expect(f3.engage).toHaveBeenCalled();
                done();
            }, function () { return fail("resolve callback should not be invoked when any of the futures fail"); });
        });
    });
    describe("gather4", function () {
        test("resolves with array of values when all 3 Futures succeed", function () {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.of("second future value");
            var f3 = Future_1.default.of("third future value");
            var f4 = Future_1.default.of("fourth future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            spyOn(f4, "engage").and.callThrough();
            Future_1.default.gather4(f1, f2, f3, f4).engage(function (e) { return fail(e); }, function (val) {
                expect(val).toBeArrayOfSize(4);
                expect(val[0]).toEqual("first future value");
                expect(val[1]).toEqual("second future value");
                expect(val[2]).toEqual("third future value");
                expect(val[3]).toEqual("fourth future value");
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                expect(f3.engage).toHaveBeenCalled();
                expect(f4.engage).toHaveBeenCalled();
            });
        });
        test("rejects with error when any subsequent future fails", function () {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.reject(new Error("second failure message"));
            var f3 = Future_1.default.of("third future value");
            var f4 = Future_1.default.reject(new Error("fourth failure message"));
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            spyOn(f4, "engage").and.callThrough();
            Future_1.default.gather4(f1, f2, f3, f4).engage(function (err) {
                expect(err).toEqual(new Error("second failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                expect(f3.engage).not.toHaveBeenCalled();
                expect(f4.engage).not.toHaveBeenCalled();
            }, function () { return fail("resolve callback should not be invoked when any of the futures fail"); });
        });
        test("engages all futures without waiting", function (done) {
            var f1 = new Future_1.default(function (reject) {
                setTimeout(function () { return reject(new Error("first failure message")); });
            });
            var f2 = new Future_1.default(function (reject) {
                setTimeout(function () { return reject(new Error("second failure message")); });
            });
            var f3 = Future_1.default.of("third future value");
            var f4 = Future_1.default.of("fourth future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            spyOn(f4, "engage").and.callThrough();
            Future_1.default.gather4(f1, f2, f3, f4).engage(function (err) {
                expect(err).toEqual(new Error("first failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                expect(f3.engage).toHaveBeenCalled();
                expect(f4.engage).toHaveBeenCalled();
                done();
            }, function () { return fail("resolve callback should not be invoked when any of the futures fail"); });
        });
    });
    describe("all", function () {
        test("resolves with array of values when all Futures succeed", function (done) {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.of("second future value");
            var f3 = Future_1.default.of("third future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            Future_1.default.all([f1, f2, f3]).engage(function () {
                fail("reject handler should not be invoked when all futures resolve");
            }, function (val) {
                expect(val).toBeArrayOfSize(3);
                expect(val[0]).toEqual("first future value");
                expect(val[1]).toEqual("second future value");
                expect(val[2]).toEqual("third future value");
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                expect(f3.engage).toHaveBeenCalled();
                done();
            });
        });
        test("rejects with error when first future fails", function (done) {
            var f1 = Future_1.default.reject(new Error("first failed future"));
            var f2 = Future_1.default.of("second future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.all([f1, f2]).engage(function (err) {
                expect(err).toEqual(new Error("first failed future"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).not.toHaveBeenCalled();
                done();
            }, function () {
                fail("resolve callback should not be invoked when any of the futures fail");
            });
        });
        test("rejects with error when any subsequent future fails", function (done) {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.reject(new Error("second failure message"));
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.all([f1, f2]).engage(function (err) {
                expect(err).toEqual(new Error("second failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                done();
            }, function () {
                fail("resolve callback should not be invoked when any of the futures fail");
            });
        });
        test("rejects with first error that occured when multiple errors", function (done) {
            var f1 = new Future_1.default(function (reject) {
                setTimeout(function () {
                    reject(new Error("first failure message"));
                });
            });
            var f2 = Future_1.default.reject(new Error("second failure message"));
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.all([f1, f2]).engage(function (err) {
                expect(err).toEqual(new Error("second failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                done();
            }, function () {
                fail("resolve callback should not be invoked when any of the futures fail");
            });
        });
        test("resolves immediately if the array is empty.", function (done) {
            Future_1.default.all([]).engage(function () {
                fail("an empty array of futures shouldn't fail");
            }, function (arr) {
                expect(arr).toEqual([]);
                done();
            });
        });
    });
    describe("allObject", function () {
        test("resolves with object with expected keys", function (done) {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.of("second future value");
            var f3 = Future_1.default.of("third future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            spyOn(f3, "engage").and.callThrough();
            Future_1.default.all({ f1: f1, f2: f2, f3: f3 }).engage(function () {
                fail("reject handler should not be invoked when all futures resolve");
            }, function (result) {
                expect(Object.keys(result)).toBeArrayOfSize(3);
                expect(result.f1).toEqual("first future value");
                expect(result.f2).toEqual("second future value");
                expect(result.f3).toEqual("third future value");
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                expect(f3.engage).toHaveBeenCalled();
                done();
            });
        });
        test("rejects with error when first future fails", function (done) {
            var f1 = Future_1.default.reject(new Error("first failed future"));
            var f2 = Future_1.default.of("second future value");
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.all({ f1: f1, f2: f2 }).engage(function (err) {
                expect(err).toEqual(new Error("first failed future"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).not.toHaveBeenCalled();
                done();
            }, function () {
                fail("resolve callback should not be invoked when any of the futures fail");
            });
        });
        test("rejects with error when any subsequent future fails", function (done) {
            var f1 = Future_1.default.of("first future value");
            var f2 = Future_1.default.reject(new Error("second failure message"));
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.all({ f1: f1, f2: f2 }).engage(function (err) {
                expect(err).toEqual(new Error("second failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                done();
            }, function () {
                fail("resolve callback should not be invoked when any of the futures fail");
            });
        });
        test("rejects with first error that occured when multiple errors", function (done) {
            var f1 = new Future_1.default(function (reject) {
                setTimeout(function () {
                    reject(new Error("first failure message"));
                });
            });
            var f2 = Future_1.default.reject(new Error("second failure message"));
            spyOn(f1, "engage").and.callThrough();
            spyOn(f2, "engage").and.callThrough();
            Future_1.default.all({ f1: f1, f2: f2 }).engage(function (err) {
                expect(err).toEqual(new Error("second failure message"));
                expect(f1.engage).toHaveBeenCalled();
                expect(f2.engage).toHaveBeenCalled();
                done();
            }, function () {
                fail("resolve callback should not be invoked when any of the futures fail");
            });
        });
        test("resolves immediately if the object is empty.", function (done) {
            Future_1.default.all({}).engage(function () {
                fail("an empty object of futures shouldn't fail");
            }, function (obj) {
                expect(obj).toEqual({});
                done();
            });
        });
    });
    describe("then-able", function () {
        test("works with await syntax.", function () { return __awaiter(void 0, void 0, void 0, function () {
            var f, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        f = Future_1.default.of(1);
                        return [4, f];
                    case 1:
                        result = _a.sent();
                        expect(result).toEqual(1);
                        return [2];
                }
            });
        }); });
        test("blows up correctly with await syntax.", function () { return __awaiter(void 0, void 0, void 0, function () {
            var f;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        f = Future_1.default.reject(new Error("bad stuff"));
                        return [4, expect(f).rejects.toThrow("bad stuff")];
                    case 1:
                        _a.sent();
                        return [2];
                }
            });
        }); });
        test("can be chained with normal promises.", function () { return __awaiter(void 0, void 0, void 0, function () {
            var p, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        p = Promise.resolve(1);
                        return [4, p.then(function (num) { return Future_1.default.of(num + 1); })];
                    case 1:
                        result = _a.sent();
                        expect(result).toEqual(2);
                        return [2];
                }
            });
        }); });
        test("blows up correctly when chained with normal promises.", function () { return __awaiter(void 0, void 0, void 0, function () {
            var p, chainedPromiseFuture;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        p = Promise.resolve(1);
                        chainedPromiseFuture = p.then(function () { return Future_1.default.reject(new Error("bad stuff")); });
                        return [4, expect(chainedPromiseFuture).rejects.toThrow("bad stuff")];
                    case 1:
                        _a.sent();
                        return [2];
                }
            });
        }); });
    });
});
